@@ -23,6 +23,7 @@
 #include <QStandardPaths>
 #include <QTest>
 
+using namespace Qt::Literals;
 using namespace KPublicTransport;
 
 class PublicTransportManagerTest : public QObject
@@ -106,6 +107,23 @@ private Q_SLOTS:
         delete reply;
     }
 
+    void testDefaultEnabledState()
+    {
+        Manager mgr;
+        const auto backends = mgr.backends();
+        if (std::ranges::none_of(backends, [](const auto &b) { return b.identifier() == "un_transitous_staging"_L1; })) {
+            QSKIP("Development backend configurations not installed.");
+        }
+        QCOMPARE(mgr.isBackendEnabled("un_transitous"_L1), true);
+        QCOMPARE(mgr.isBackendEnabled("un_transitous_staging"_L1), false);
+        mgr.setBackendsEnabledByDefault(false);
+        QCOMPARE(mgr.isBackendEnabled("un_transitous"_L1), false);
+        QCOMPARE(mgr.isBackendEnabled("un_transitous_staging"_L1), false);
+        mgr.setBackendsEnabledByDefault(true);
+        QCOMPARE(mgr.isBackendEnabled("un_transitous"_L1), true);
+        QCOMPARE(mgr.isBackendEnabled("un_transitous_staging"_L1), false);
+    }
+
     void testReload()
     {
         Manager mgr;
