@@ -11,6 +11,9 @@
 #include <KPublicTransport/RentalVehicle>
 #include <KPublicTransport/Stopover>
 
+#include <KCountry>
+#include <KCountrySubdivision>
+
 #include <QColor>
 #include <QDebug>
 #include <QJsonArray>
@@ -83,6 +86,27 @@ struct {
     { "TRANSIT", Location::Stop },
 };
 
+constexpr inline auto PICKUP_DROPOFF_NOT_ALLED = "NOT_ALLOWED"_L1;
+
+template <typename T>
+void parseAlerts(T &elem, const QJsonArray &alerts)
+{
+    for (const auto &alertV : alerts) {
+        const auto alertObj = alertV.toObject();
+        const auto headline = alertObj.value("headerText"_L1).toString();
+        const auto desc = alertObj.value("descriptionText"_L1).toString();
+        if (!headline.isEmpty() && !desc.isEmpty()) {
+            elem.addNote("<b>"_L1 + headline + "</b><br/>"_L1 + desc);
+        } else {
+            elem.addNote(headline);
+            elem.addNote(desc);
+        }
+        if (alertObj.value("effect"_L1).toString() == "NO_SERVICE"_L1) {
+            elem.setDisruptionEffect(Disruption::NoService);
+        }
+    }
+}
+
 Stopover Motis2Parser::parsePlace(const QJsonObject &obj, bool hasRealTime) const
 {
     Location l;
@@ -113,7 +137,15 @@ Stopover Motis2Parser::parsePlace(const QJsonObject &obj, bool hasRealTime) cons
     if (obj.value("cancelled"_L1).toBool()) {
         s.setDisruptionEffect(Disruption::NoService);
     }
-    // TODO dropoffType, pickupType
+
+    if (obj.value("pickupType"_L1).toString() == PICKUP_DROPOFF_NOT_ALLED) {
+        s.setPickupType(PickupDropoff::NotAllowed);
+    }
+    if (obj.value("dropoffType"_L1).toString() == PICKUP_DROPOFF_NOT_ALLED) {
+        s.setDropoffType(PickupDropoff::NotAllowed);
+    }
+
+    parseAlerts(s, obj.value("alerts"_L1).toArray());
 
     return s;
 }
@@ -216,6 +248,8 @@ Journey Motis2Parser::parseItinerary(const QJsonObject &itinerary) const
             if (leg.value("cancelled"_L1).toBool()) {
                 s.setDisruptionEffect(Disruption::NoService);
             }
+
+            parseAlerts(s, leg.value("alerts"_L1).toArray());
         } else if (s.mode() == JourneySection::IndividualTransport) {
             IndividualTransport iv;
             iv.setMode((*it).ivMode);
@@ -290,7 +324,7 @@ Journey Motis2Parser::parseItinerary(const QJsonObject &itinerary) const
     return jny;
 }
 
-std::vector<Stopover> Motis2Parser::parseStopTimes(const QByteArray &data)
+std::vector<Stopover> Motis2Parser::parseStopTimes(const QByteArray &data, bool isArrivals)
 {
     std::vector<Stopover> result;
 
@@ -317,7 +351,16 @@ std::vector<Stopover> Motis2Parser::parseStopTimes(const QByteArray &data)
 
         const auto hasRealTime = stop.value("realTime"_L1).toBool();
         auto s = parsePlace(stop.value("place"_L1).toObject(), hasRealTime);
+        s.setTripIdentifier(m_locIdentifierType, stop.value("tripId"_L1).toString());
         s.setRoute(route);
+
+        if (stop.value("cancelled"_L1).toBool()) {
+            s.setDisruptionEffect(Disruption::NoService);
+        }
+        if (stop.value("pickupDropoffType"_L1).toString() == PICKUP_DROPOFF_NOT_ALLED) {
+            isArrivals ? s.setDropoffType(PickupDropoff::NotAllowed) : s.setPickupType(PickupDropoff::NotAllowed);
+        }
+
         result.push_back(std::move(s));
     }
 
@@ -334,6 +377,17 @@ struct {
     { "STOP", Location::Stop },
 };
 
+// see https://github.com/osm-search/Nominatim/blob/master/settings/address-levels.json
+constexpr inline auto DEFAULT_REGION_LEVEL = 4;
+struct {
+    const char code[3];
+    uint8_t level;
+} static constexpr const region_admin_level_map[] = {
+    { "BE", 6 },
+    { "NO", 0 },
+    { "SE", 0 },
+};
+
 std::vector<Location> Motis2Parser::parseLocations(const QByteArray &data) const
 {
     const auto locations = QJsonDocument::fromJson(data).array();
@@ -356,19 +410,50 @@ std::vector<Location> Motis2Parser::parseLocations(const QByteArray &data) const
         l.setLatitude(locObj.value("lat"_L1).toDouble());
         l.setLongitude(locObj.value("lon"_L1).toDouble());
         l.setFloorLevel(locObj.value("level"_L1).toInt(std::numeric_limits<int>::lowest()));
+
         const auto areas = locObj.value("areas"_L1).toArray();
-        int cityLevel = std::numeric_limits<int>::max();
+        std::vector<QJsonObject> adminLevels(12);
         for (const auto &areaV : areas) {
             const auto area = areaV.toObject();
-            if (const auto level = area.value("adminLevel"_L1).toInt(); level >= 8 && level < cityLevel) {
-                // TODO needs a proper country-specific admin-level mapping, for now taken from Motis v1 parser
-                // see https://wiki.openstreetmap.org/wiki/Key:admin_level
-                cityLevel = level;
-                l.setLocality(area.value("name"_L1).toString());
+            const auto level = area.value("adminLevel"_L1).toInt();
+            if (level < 0 || level >= (int)adminLevels.size()) {
+                continue;
+            }
+            adminLevels[level] = area;
+        }
+
+        // resolve country
+        // we need to do this here rather than relying on post-processing as the subsequent admin level mapping depends on it
+        const auto countryName = adminLevels[2].value("name"_L1).toString();
+        auto country = KCountry::fromName(countryName);
+        if (!country.isValid()) {
+            country = KCountry::fromLocation((float)l.latitude(), (float)l.longitude());
+        }
+        l.setCountry(country.isValid() ? country.alpha2() : countryName);
+
+        // resolve region
+        if (const auto r = KCountrySubdivision::fromLocation((float)l.latitude(), (float)l.longitude()); r.isValid()) {
+            l.setRegion(r.code());
+        }
+        if (country.isValid() && l.region().isEmpty()) {
+            const auto it = std::find_if(std::begin(region_admin_level_map), std::end(region_admin_level_map), [country](const auto &m) {
+                return QLatin1StringView(m.code, 2) == country.alpha2();
+            });
+            l.setRegion(adminLevels[it != std::end(region_admin_level_map) ? (*it).level :DEFAULT_REGION_LEVEL].value("name"_L1).toString());
+        }
+
+        // city name
+        // TODO needs a proper country-specific admin-level mapping, for now taken from Motis v1 parser
+        // see https://wiki.openstreetmap.org/wiki/Key:admin_level
+        for (int i = 8; i < (int)adminLevels.size(); ++i) {
+            if (const auto n = adminLevels[i].value("name"_L1).toString(); !n.isEmpty()) {
+                l.setLocality(n);
+                break;
             }
         }
+
         l.setPostalCode(locObj.value("zip"_L1).toString());
-        l.setStreetAddress(locObj.value("street"_L1).toString() + ' '_L1 + locObj.value("houseNumber"_L1).toString());
+        l.setStreetAddress(QString(locObj.value("street"_L1).toString() + ' '_L1 + locObj.value("houseNumber"_L1).toString()).trimmed());
         result.push_back(std::move(l));
     }
 
