@@ -13,6 +13,7 @@
 #include "locationreply.h"
 #include "locationrequest.h"
 #include "logging.h"
+#include "standardpaths_p.h"
 #include "stopoverreply.h"
 #include "stopoverrequest.h"
 #include "tripreply.h"
@@ -25,6 +26,8 @@
 #include "datatypes/disruption.h"
 #include "datatypes/json_p.h"
 #include "geo/geojson_p.h"
+#include "update/updatejob_p.h"
+#include "update/updatestate_p.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Location>
@@ -58,6 +61,7 @@
 #include <QTimeZone>
 
 #include <functional>
+#include <unordered_set>
 
 using namespace Qt::Literals::StringLiterals;
 using namespace KPublicTransport;
@@ -105,6 +109,8 @@ public:
     std::vector<Backend> m_backends;
     std::vector<Attribution> m_attributions;
 
+    // backends that are configured to be off by default, regardless of m_backendsEnabledByDefault
+    QStringList m_disabledByDefault;
     // we store both explicitly to have a third state, backends with the enabled state being the "default" (whatever that might eventually be)
     QStringList m_enabledBackends;
     QStringList m_disabledBackends;
@@ -113,6 +119,8 @@ public:
     bool m_hasReadCachedAttributions = false;
     bool m_backendsEnabledByDefault = true;
 
+    UpdateResult::Type m_updateResult = UpdateResult::NoError;
+
 private:
     [[nodiscard]] bool shouldSkipBackend(const Backend &backend) const;
 };
@@ -136,20 +144,16 @@ void ManagerPrivate::loadNetworks()
         return;
     }
 
-    QStringList searchDirs;
-#ifndef Q_OS_ANDROID
-    searchDirs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
-#endif
-    searchDirs.push_back(u":/"_s);
-
+    const auto searchDirs = StandardPaths::dataSearchPaths();
     std::vector<Attribution> attributions;
+    std::unordered_set<QString> loadedBackends;
     for (const auto &searchDir : searchDirs) {
-        QDirIterator it(searchDir + "/org.kde.kpublictransport/networks"_L1, {u"*.json"_s}, QDir::Files);
+        QDirIterator it(searchDir + "/networks"_L1, {u"*.json"_s}, QDir::Files);
         while (it.hasNext()) {
             it.next();
             const auto id = it.fileInfo().baseName();
-            if (std::any_of(m_backends.begin(), m_backends.end(), [&id](const auto &backend) { return backend.identifier() == id; })) {
-                // already seen in another location
+            if (loadedBackends.contains(id)) {
+                // already loaded from an earlier location
                 continue;
             }
 
@@ -165,8 +169,16 @@ void ManagerPrivate::loadNetworks()
                 qCWarning(Log) << "Failed to parse public transport network configuration:" << error.errorString() << it.fileName();
                 continue;
             }
+            const auto obj = doc.object();
 
-            auto net = loadNetwork(doc.object());
+            if (!obj.value("active"_L1).toBool(true)) {
+                // disabled in configuration, also skip all later instances of this then!
+                qCDebug(Log) << "Not loading deactivated backend:" << it.fileName();
+                loadedBackends.insert(id);
+                continue;
+            }
+
+            auto net = loadNetwork(obj);
             if (net) {
                 net->setBackendId(id);
                 net->init();
@@ -177,15 +189,20 @@ void ManagerPrivate::loadNetworks()
                 auto b = BackendPrivate::fromJson(doc.object());
                 BackendPrivate::setImpl(b, std::move(net));
                 m_backends.push_back(std::move(b));
+                loadedBackends.insert(id);
+                if (!obj.value("enabled"_L1).toBool(true)) {
+                    m_disabledByDefault.push_back(id);
+                }
             } else {
                 qCWarning(Log) << "Failed to load public transport network configuration config:" << it.fileName();
             }
         }
     }
 
-    std::stable_sort(m_backends.begin(), m_backends.end(), [](const auto &lhs, const auto &rhs) {
+    std::ranges::stable_sort(m_backends, [](const auto &lhs, const auto &rhs) {
         return lhs.identifier() < rhs.identifier();
     });
+    std::ranges::sort(m_disabledByDefault);
 
     AttributionUtil::sort(attributions);
     if (m_attributions.empty()) {
@@ -507,6 +524,8 @@ Manager::Manager(QObject *parent)
     }
 
     Cache::expire();
+    UpdateState::purgeObsoleteFiles();
+    d->m_updateResult = UpdateState::isEnabled() ? UpdateResult::NoError : UpdateResult::UpdatesDisabled;
 
     QCoreApplication::instance()->installEventFilter(this);
 }
@@ -778,7 +797,7 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
                 continue;
             }
             const auto coverage = backend.coverageArea(coverageType);
-            if (coverage.isEmpty() || !coverage.coversLocation(loc)) {
+            if (coverage.isEmpty() || !coverage.coversLocation(loc) || !coverage.coversArea(req.viewbox())) {
                 continue;
             }
             if (isCountryOnly && !coverage.hasNationWideCoverage(loc.country())) {
@@ -799,7 +818,7 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
                 continue;
             }
             const auto coverage = backend.coverageArea(coverageType);
-            if (coverage.isEmpty() || !coverage.coversLocation(loc)) {
+            if (coverage.isEmpty() || !coverage.coversLocation(loc) || !coverage.coversArea(req.viewbox())) {
                 continue;
             }
 
@@ -821,9 +840,10 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
 }
 
 // Check whether a given JourneySection matches a trip request in fallback journey query mode
-// This is essentially the same as JourneySection::isSame, with one special case for accepting
-// all specialized modes for an requested generic mode. Line::isSame takes care of most of this
-// on its own already, we just need to handle all rail-based modes explicitly here.
+// This is essentially the same as JourneySection::isSame, with a few special cases:
+// * accepting all specialized modes for an requested generic mode. Line::isSame takes care of most of this
+//   on its own already, we just need to handle all rail-based modes explicitly here.
+// * being more tolerant towards moved departure times if the route is assumed to be unique during a day
 [[nodiscard]] static bool isSameTrip(KPublicTransport::JourneySection req, const KPublicTransport::JourneySection &res)
 {
     if (req.route().line().mode() == Line::Train && Line::modeIsRailBound(res.route().line().mode())) {
@@ -833,6 +853,12 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
         route.setLine(line);
         req.setRoute(route);
     }
+    if (Route::isSame(req.route(), res.route()) && res.route().line().mode() == Line::Mode::LongDistanceTrain) {
+        const auto depDelta = std::abs(req.scheduledDepartureTime().secsTo(res.scheduledDepartureTime()));
+        if (depDelta < 30l * 60) {
+            req.setScheduledDepartureTime(res.scheduledDepartureTime());
+        }
+    }
     return JourneySection::isSame(req, res);
 }
 
@@ -850,54 +876,22 @@ TripReply* Manager::queryTrip(const TripRequest &req) const
 
     d->loadNetworks();
 
-    // try to find a viable backend that can do a trip query directly
-    QSet<QString> triedBackends;
-    for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
-        const auto checkBackend = [&](const Backend &backend, bool bothLocationMatch) {
-            if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
-                return;
+    // no need for coverage-based checks here, queryTrip() is identifier-based
+    // we only need coverage-based backend selection for the journey query fallback,
+    // but queryJourney as called below does that for us
+    if (req.hasIdentifiers()) {
+        for (const auto &backend : d->m_backends) {
+            if (d->shouldSkipBackend(backend, req)) {
+                continue;
             }
-            const auto coverage = backend.coverageArea(coverageType);
-            if (coverage.isEmpty()) {
-                return;
-            }
-
-            if (bothLocationMatch) {
-                if (!coverage.coversLocation(req.journeySection().from()) || !coverage.coversLocation(req.journeySection().to())) {
-                    return;
-                }
-            } else {
-                if (!coverage.coversLocation(req.journeySection().from()) && !coverage.coversLocation(req.journeySection().to())) {
-                    return;
-                }
-            }
-
-            triedBackends.insert(backend.identifier());
-
             if (BackendPrivate::impl(backend)->queryTrip(req, reply, d->nam())) {
                 ++pendingOps;
             }
-        };
-
-        // look for coverage areas which contain both locations first
-        for (const auto &backend: d->m_backends) {
-            checkBackend(backend, true);
-        }
-        if (pendingOps) {
-            break;
-        }
-
-        // if we didn't find one, try with just a single one
-        for (const auto &backend: d->m_backends) {
-            checkBackend(backend, false);
-        }
-        if (pendingOps) {
-            break;
         }
     }
 
     // emulate a trip query via a journey query
-    if (pendingOps == 0) {
+    if (pendingOps == 0 && req.journeySection().mode() != JourneySection::Invalid) {
         JourneyRequest jnyReq(req.journeySection().from(), req.journeySection().to());
         // start searching slightly earlier, so leading walking section because our coordinates
         // aren't exactly at the right spot wont make the routing service consider the train we
@@ -1040,6 +1034,9 @@ const std::vector<Backend>& Manager::backends() const
 
 bool Manager::isBackendEnabled(const QString &backendId) const
 {
+    d->loadNetworks();
+
+    // ### should be std::ranges::binary_search, but that produces wrong results with Android NDK r26...
     if (std::binary_search(d->m_disabledBackends.cbegin(), d->m_disabledBackends.cend(), backendId)) {
         return false;
     }
@@ -1047,7 +1044,7 @@ bool Manager::isBackendEnabled(const QString &backendId) const
         return true;
     }
 
-    return d->m_backendsEnabledByDefault;
+    return d->m_backendsEnabledByDefault && !std::binary_search(d->m_disabledByDefault.cbegin(), d->m_disabledByDefault.cend(), backendId);
 }
 
 static void sortedInsert(QStringList &l, const QString &value)
@@ -1116,15 +1113,6 @@ void Manager::setBackendsEnabledByDefault(bool byDefault)
     Q_EMIT configurationChanged();
 }
 
-QVariantList Manager::backendsVariant() const
-{
-    d->loadNetworks();
-    QVariantList l;
-    l.reserve(d->m_backends.size());
-    std::transform(d->m_backends.begin(), d->m_backends.end(), std::back_inserter(l), [](const auto &b) { return QVariant::fromValue(b); });
-    return l;
-}
-
 bool Manager::eventFilter(QObject *object, QEvent *event)
 {
     if (event->type() == QEvent::LanguageChange && object == QCoreApplication::instance() && !QCoreApplication::closingDown()) {
@@ -1134,4 +1122,37 @@ bool Manager::eventFilter(QObject *object, QEvent *event)
     return QObject::eventFilter(object, event);
 }
 
+void Manager::checkForUpdates(bool force)
+{
+    if (d->m_updateResult == UpdateResult::InProgress) {
+        return;
+    }
+
+    if (!force && UpdateState::lastUpdate().addDays(1) > QDateTime::currentDateTimeUtc()) {
+        d->m_updateResult = UpdateResult::NoUpdate;
+        Q_EMIT updateResultChanged();
+        return;
+    }
+
+    d->m_updateResult = UpdateResult::InProgress;
+    Q_EMIT updateResultChanged();
+
+    auto job = new UpdateJob(this);
+    job->start(d->nam());
+    connect(job, &UpdateJob::finished, this, [this, job]() {
+        job->deleteLater();
+        qCDebug(Log) << job->result();
+        d->m_updateResult = job->result();
+        if (job->result() == UpdateResult::UpdateSuccessful) {
+            reload();
+        }
+        Q_EMIT updateResultChanged();
+    });
+}
+
+UpdateResult::Type Manager::updateResult() const
+{
+    return d->m_updateResult;
+}
+
 #include "moc_manager.cpp"
